#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
test_brainiac
----------------------------------

tests for `brainiac` module.
"""

import unittest
import cloudpickle
from memex_client.scientist import package


class MockModel:
    def __init__(self, val: int):
        self.some_value = val

    def some_function(self, number: int):
        orig_some_val = self.some_value
        self.some_value = number
        return orig_some_val + number


class TestScientist(unittest.TestCase):
    def test_package(self):
        serialized_model = package(MockModel(42))
        model = cloudpickle.loads(serialized_model)
        assert model.some_value == 42
        assert model.some_function(1) == 43
        assert model.some_value == 1



from io import BytesIO
from typing import Any
import os
from memex_client.client import BRAINIAC_CLIENT_PASSWORD, BRAINIAC_CLIENT_USERNAME
import cloudpickle
import requests


def login(url, username: str, password: str):
    login_url = "{}/api/login".format(url)
    params = {'username': username, 'password': password}
    res = requests.post(url=login_url, json=params)
    return res


def package(some_class: Any, modelName: str,
            url: str = "http://localhost:4000") -> requests.Response:

    res = login(url, os.environ.get(BRAINIAC_CLIENT_USERNAME),
                os.environ.get(BRAINIAC_CLIENT_PASSWORD))
    cookies = res.headers.get("Set-Cookie")

    file = BytesIO(cloudpickle.dumps(some_class))

    files = {"pkledFile": file}
    params = {"modelName": modelName}
    url = url + "/api/model/upload"
    response = requests.put(url, files=files, data=params, headers={"Cookie": cookies})
    return response


__all__ = ["package"]



import atexit
import os
from typing import Dict, List, Optional

import grpc
from google.protobuf.struct_pb2 import Struct

from memex_client.protos.brainiac_pb2 import (BrainiacRequest, ProjectData, ThirdPartyRequest,
                                              UserData)
from memex_client.protos.brainiac_pb2_grpc import EngineStub
from memex_client.errors import AuthenticationError

BRAINIAC_CLIENT_USERNAME = "BRAINIAC_CLIENT_USERNAME"
BRAINIAC_CLIENT_PASSWORD = "BRAINIAC_CLIENT_PASSWORD"
DEFAULT_PORT = 5000


class ModelClient:
    def __init__(self, grpc_stub, client, project, force_download):
        self.stub = grpc_stub
        self.client = client
        self.project = project
        self.force_download = force_download

    def set_methods(self, methods: List[str]):
        [setattr(self, method, self.__create_method(method)) for method in methods]

    def __create_method(self, name: str):
        def wrapper(body: Dict):
            struct = Struct()
            struct.update(body)
            response = self.stub.runFunction(
                BrainiacRequest(client=self.client,
                                project=self.project,
                                function_name=name,
                                body=struct,
                                force_download=self.force_download))
            if response.status.status_code == 200:
                return response.body
            return None

        return wrapper


class BrainiacClient:
    def __init__(self, url: Optional[str] = None, stub: EngineStub = None):
        if url is None:
            url = 'localhost:{}'.format(DEFAULT_PORT)
        self.channel = grpc.insecure_channel(url)
        self.stub = stub or EngineStub(self.channel)

        self.client = UserData(username=os.environ.get(BRAINIAC_CLIENT_USERNAME),
                               password=os.environ.get(BRAINIAC_CLIENT_PASSWORD))
        atexit.register(self.channel.close)

    def get_model_client(self, creator: str, project: str,
                         force_download=False) -> Optional[ModelClient]:
        project_data = ProjectData(creator=creator, name=project)
        response = self.stub.listFunctions(BrainiacRequest(client=self.client,
                                                           project=project_data))
        if response.status.status_code == 200:
            methods = response.body["response"]
            model = ModelClient(self.stub, self.client, project_data, force_download)
            model.set_methods(methods)
            return model
        elif response.status.status_code == 404:
            return None
        elif response.status.status_code == 401:
            raise AuthenticationError(response.status.status_reason)
        else:
            raise Exception(response.status.status_reason)

    def runThirdPartyFunction(self,
                              models: ModelClient,
                              module_name,
                              method_name,
                              data: Dict,
                              force_download=False) -> Dict:
        ThirdPartyRequest()
        body = Struct()
        body.update(data)
        response = self.stub.thirdPartyFunction(
            ThirdPartyRequest(client=self.client,
                              projects=[model.project for model in models],
                              function_name=method_name,
                              module=module_name,
                              body=body,
                              force_download=force_download))
        if response.status.status_code == 200:
            return response.body
        return None



# -*- coding: utf-8 -*-
from __future__ import print_function
from ._version import get_versions

__author__ = 'Suyog Soti'
__email__ = 'suyog.soti@gmail.com'
__version__ = get_versions()['version']
del get_versions
